import sys
import time
import logging
import dask.dataframe as dd
import pandas as pd
import sqlite3
import os
from watchdog.observers.polling import PollingObserver as Observer
from watchdog.events import FileSystemEventHandler

class MyQueue(object):
    """
    A queue for handling integers.

    ...

    Attributes
    ----------
    queue : list
        A list of integers representing rows in a dataframe.
    """
    queue = []

    def enqueue(self, element):
        """Enque an element into the queue.

        Parameters
        ----------
        element : int or list of int
            If an int it is the element to be enqued into the queue
            if a list of ints the list is appended to the queue
        """
        if isinstance(element, list):
            self.queue.extend(element)
        else:
            self.queue.append(element)

    def dequeue(self):
        """Dequeue an element from the queue.

        Returns
        ----------
        int or list of int
            the zeroth element in the queue list
        """
        if len(self.queue) == 0:
            return None
        else:
            return self.queue.pop(0) #pop(0) specifies to pop the zeroth list element

class MyCSVHandler(FileSystemEventHandler):
    """
    An event handler to watch the csv.

    ...

    Attributes
    ----------
    currentRow : int
        The next row to be processed
    queue : list
        A list of integers representing rows in a dataframe.
    currentDf : DataFrame
        The csv as it was last read in
    """
    currentRow = 0
    queue = MyQueue()

    def __init__(self, row, queue):
        """Initialize the myCsv handler.

        Parameters
        ----------
        row : int or list of int
            The index of the first row to begin processing at
        queue : MyQueue
            The queue in which to store row indexes in
        """
        self.currentRow = row
        self.queue = queue

    def on_created(self, event):
        """Prints the file path if the file being watched for is created.

        Parameters
        ----------
        event : event
            The event object representing the event that triggered the handler.
        """
        print("Watchdog received created event - % s." % event.src_path)
        # Event is created, you can process it now

    def on_modified(self, event):
        """Prints the file path if the file being watched for is modified and adds the indecies of all
        rows in the csv including and after "self.currentRow".

        Parameters
        ----------
        event : event
            The event object representing the event that triggered the handler.
        """
        print("Watchdog received modified event - % s." % event.src_path)
        currentDf = pd.read_csv(event.src_path)
        lastRow = currentDf.shape[0]
        rowSpan = get_row_span(self.currentRow, lastRow)
        #print('rowspan: ')
        #print(rowSpan)
        self.queue.enqueue(rowSpan)
        self.currentRow = lastRow
        # Event is modified, youu\ can process it now

# class MyZipHandler(FileSystemEventHandler):
#     queue = []

#     def __init__(self, queue):
#         """Initialize the MyZipHandler.

#         Parameters
#         ----------
#         queue : MyQueue
#             The queue in which to store row indexes in
#         """
#         self.queue = queue

#     def on_created(self, event):
#         """Prints the file path if the file being watched for is created and adds the indecies of all.

#         Parameters
#         ----------
#         event : event
#             The event object representing the event that triggered the handler.
#         """
#         print("Watchdog received created event - % s." % event.src_path)
#         # Event is created, you can process it now

#     def on_modified(self, event):
#         """Prints the file path if the file being watched is modified and adds the path to a queue.

#         Parameters
#         ----------
#         event : event
#             The event object representing the event that triggered the handler.
#         """
#         print("Watchdog received modified event - % s." % event.src_path)
#         self.queue.append(event.src_path)
#         # Event is modified, youu\ can process it now


def consume(filePath, mntPath, line, db):
    """Processes neurocube jobs by watching the gdfuse file system.

    Parameters
    ----------
    filePath : str
        The path of the file or directory to watch for changes.
    line : int
        If processing a csv file the last line in the frame that
        has already been processed.
    Notes
    -----
    Ctrl-C will terminate the function.
    """
    queue = MyQueue()
    df = pd.DataFrame()
    event_handler = MyCSVHandler(line, queue)
    observer = Observer()
    observer.schedule(event_handler, filePath, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1) #This may not be needed
            currentJob = queue.dequeue()
            if currentJob != None:
                print(df)
                dummy_process(currentJob, df, mntPath, db, filePath)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()

def get_row_span(first,last):
    """Returns a list of integers (first,last].

    Parameters
    ----------
    first : int
        the integer before the first element in the list.
    last : int
        the last element in the list
    Returns
    -------
    list of int
        a list of integers spanning the next integer greeater than first to last
    """
    if first == last:
        return [last]
    else:
        return list(range(first,last))

def dummy_process(job, df, mntPath, db, filePath):
    """Prints a row from the csv file.

    Parameters
    ----------
    mountPoint : str
        The directory to mount the google drive to
    """
    df = pd.read_csv(filePath)
    row = get_row(job,df)
    gdUrl = row['magicVal'] #TODO: this column name needs to be added to the csv MANUALLY
    GId = parse_gdUrl(gdUrl)
    print(GId)
    path = gid_to_path(GId, mntPath, db)
    print('dummy: ')
    print(job)
    print(row)
    print(path)

def get_row(job,df):
    """Prints a row from the csv file.

    Parameters
    ----------
    mountPoint : str
        The directory to mount the google drive to
    """
    return df.iloc[job]

def gid_to_path(GId, mntPath, db):
    conn = sqlite3.connect(db)
    c = conn.cursor()
    relPath = get_relPath(GId, c)
    path = os.path.join(mntPath, relPath)
    conn.close()
    return path

def get_relPath(GId, c):
   while True:
       c.execute("""SELECT path FROM resource WHERE remote_id=:remote_id""", {'remote_id': GId})
       relPath = c.fetchone()
       if relPath != None:
           break
       print('miss')
   return relPath[0]

def parse_gdUrl(gdUrl):
    return gdUrl.split("id=",1)[1]
